﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO; //MemoryStream
using Microsoft.Win32.SafeHandles;

namespace RX62N_cs
{
    class Program
    {
        static void Main(string[] args)
        {
            RX62Function echo = new RX62Function();
            echo.Run();
#if DEBUG
            Console.WriteLine("Enter to exit.");
            Console.ReadLine();
#endif
        }
    }

    
    internal class RX62Function
    {
        //  This GUID must match the GUID in the device's INF file.
        //  To create a GUID in Visual Studio, click Tools > Create GUID.

//  DEFINE_GUID(GUID_DEVINTERFACE_RX62N, // Generated using guidgen.exe
//  0x94a23b10, 0x6d99, 0x4903, 0xad, 0x6e, 0x40, 0x42, 0xd7, 0x23, 0xe4, 0xab);
//  {94A23B10-6D99-4903-AD6E-4042D723E4AB}

//      private const String USB_GUID_STRING = "{573E8C73-0CB4-4471-A1BF-FAB26C31D384}";
        private const String USB_GUID_STRING = "{94A23B10-6D99-4903-AD6E-4042D723E4AB}";
        private DeviceManagement myDeviceManagement = new DeviceManagement();
        private String myDevicePathName;
        private UsbDevice myUsbDevice = new UsbDevice();

        private delegate void ReadFromDeviceDelegate
            (Byte pipeID,
            UInt32 bufferLength,
            ref Byte[] buffer,
            ref UInt32 lengthTransferred,
            ref Boolean success);

        //internal RX62Function();

        internal void Run()
        {
            Boolean deviceFound;
            String devicePathName = "";
            Boolean success;

            try
            {
                //  Convert the device interface GUID String to a GUID object: 

                System.Guid UsbGuid =
                        new System.Guid(USB_GUID_STRING);

                // Fill an array with the device path names of all attached devices with matching GUIDs.

                deviceFound = myDeviceManagement.FindDeviceFromGuid
                    (UsbGuid,
                    ref devicePathName);

                if (!deviceFound)
                {
                    Console.WriteLine("Device GUID is not matched:" + USB_GUID_STRING);
                    return;
                }

                success = myUsbDevice.GetDeviceHandle(devicePathName);
                if (success)
                {
                    Console.WriteLine("Device detected:" + devicePathName);

                    // Save DevicePathName so OnDeviceChange() knows which name is my device.

                    myDevicePathName = devicePathName;
                }
                else
                {
                    // There was a problem in retrieving the information.
                    Console.WriteLine("Device is not detected:" + devicePathName);

                    myUsbDevice.CloseDeviceHandle();

                    return;
                }

                Test();

                myUsbDevice.CloseDeviceHandle();

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                throw;
            }
        }

        private void Test()
        {
            int count = 1;
            byte[] buffer = new byte[1];

            Console.WriteLine("Test start.\n");

            count = myUsbDevice.Read(buffer, 1);
            Console.WriteLine("Read successfully = {0}\n", count);

            int pattern = 0x01;
            for (int i = 0; i < 8; i++)
            {
                Console.WriteLine("SW[{0}] = {1}\n",
                    i, (buffer[0] & pattern) != 0 ? "ON" : "OFF");
                pattern <<= 1;
            }

            buffer[0] = 0xC5;
            count = myUsbDevice.Write(buffer, 1);
            Console.WriteLine("Write successfully = {0}\n", count);
        }
    }

    sealed internal partial class UsbDevice
    {
        internal struct devInfo
        {
            internal SafeFileHandle deviceHandle;
            internal FileStream usbDevice;
            internal BinaryReader reader;
            internal BinaryWriter writer;
        }

        internal devInfo myDevInfo = new devInfo();

        internal int Read(byte[] buffer, int count)
        {
            int bytes = -1;

            try
            {
                if (!(myDevInfo.deviceHandle.IsInvalid))
                {
                    bytes = myDevInfo.reader.Read(buffer, 0, count);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                throw;
            }
            return bytes;
        }

        internal int Write(byte[] buffer, int count)
        {
            int bytes = -1;

            try
            {
                if (!(myDevInfo.deviceHandle.IsInvalid))
                {
                    myDevInfo.writer.Write(buffer, 0, count);
                    bytes = count;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                throw;
            }
            return bytes;
        }

        internal void CloseDeviceHandle()
        {
            try
            {
                if (!(myDevInfo.deviceHandle.IsInvalid))
                {
                    myDevInfo.usbDevice.Close();
                    myDevInfo.deviceHandle.Close();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                throw;
            }
        }

        ///  <summary>
        ///  Requests a handle with CreateFile.
        ///  </summary>
        ///  
        ///  <param name="devicePathName"> Returned by SetupDiGetDeviceInterfaceDetail 
        ///  in an SP_DEVICE_INTERFACE_DETAIL_DATA structure. </param>
        ///  
        ///  <returns>
        ///  The handle.
        ///  </returns>

        internal Boolean GetDeviceHandle(String devicePathName)
        {
            // ***
            // API function

            //  summary
            //  Retrieves a handle to a device.

            //  parameters 
            //  Device path name returned by SetupDiGetDeviceInterfaceDetail
            //  Type of access requested (read/write).
            //  FILE_SHARE attributes to allow other processes to access the device while this handle is open.
            //  Security structure. Using Null for this may cause problems under Windows XP.
            //  Creation disposition value. Use OPEN_EXISTING for devices.
            //  Flags and attributes for files. The winsub driver requires FILE_FLAG_OVERLAPPED.
            //  Handle to a template file. Not used.

            //  Returns
            //  A handle or INVALID_HANDLE_VALUE.
            // ***

            myDevInfo.deviceHandle = FileIO.CreateFile
                (devicePathName,
                (FileIO.GENERIC_WRITE | FileIO.GENERIC_READ),
                FileIO.FILE_SHARE_READ | FileIO.FILE_SHARE_WRITE,
                IntPtr.Zero,
                FileIO.OPEN_EXISTING,
                //FileIO.FILE_ATTRIBUTE_NORMAL | FileIO.FILE_FLAG_OVERLAPPED,
                FileIO.FILE_ATTRIBUTE_NORMAL,
                0);

            if (!(myDevInfo.deviceHandle.IsInvalid))
            {

                myDevInfo.usbDevice = new FileStream(myDevInfo.deviceHandle, FileAccess.ReadWrite);
                if (myDevInfo.usbDevice != null)
                {
                    myDevInfo.reader = new BinaryReader(myDevInfo.usbDevice);
                    myDevInfo.writer = new BinaryWriter(myDevInfo.usbDevice);
                }

                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
